// Utility functions for filtering proxies

#define _GNU_SOURCE

#include "checks.h"

#include <pwd.h>
#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <exim4-signat/communication.h>

char *signat_conf_dir(void) {
	size_t buflen = sysconf(_SC_GETPW_R_SIZE_MAX);
	if (buflen == (size_t) -1)
		return NULL;

	char *buf = malloc(buflen);
	if (!buf)
		return NULL;

	struct passwd pwbuf;
	struct passwd *pwbufp;
	getpwuid_r(geteuid(), &pwbuf, buf, buflen, &pwbufp);
	if (!pwbufp)  {
		free(buf);
		return NULL;
	}

	char *conf_dir;
	bool error = asprintf(&conf_dir, "%s/.exim4-signat", pwbuf.pw_dir)
		== -1;
	free(buf);
	if (error)
		return NULL;

	return conf_dir;
}

char signat_reject_code(void) {
	char *dir = signat_conf_dir();
	if (!dir)
		return RESULT_FLAG;

	char *path;
	if (asprintf(&path, "%s/reject", dir) == -1) {
		free(dir);
		return RESULT_FLAG;
	}

	bool reject = !euidaccess(path, F_OK);
	free(path);
	if (reject) {
		free(dir);
		return RESULT_REJECT;
	}

	bool error = asprintf(&path, "%s/fake-reject", dir) == -1;
	free(dir);
	if (error)
		return RESULT_FLAG;

	bool fake_reject = !euidaccess(path, F_OK);
	free(path);
	if (fake_reject)
		return RESULT_FAKE_REJECT;

	return RESULT_FLAG;
}
