// S/MIME check

#include "checks.h"

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>

#include "smime_check.h"

static bool skip_prologue(FILE *stream, const char *mm_boundary);

bool signat_smime_check(int content_fd, const char *sender, const char *micalg,
	const char *boundary) {
	static const size_t MAX_BOUNDARY_SIZE = 71;
	size_t boundary_length = strnlen(boundary, MAX_BOUNDARY_SIZE);
	if (!boundary_length || boundary_length == MAX_BOUNDARY_SIZE)
		return true;

	boundary_length++;
	char *mm_boundary = malloc(boundary_length + 2);
	if (!mm_boundary)
		return true;

	char *boundary_p = mm_boundary;
	*boundary_p++ = '-';
	*boundary_p++ = '-';
	memcpy(boundary_p, boundary, boundary_length);

	off_t offset = lseek(content_fd, 0, SEEK_CUR);
	if (offset == -1) {
		free(mm_boundary);
		return true;
	}

	int fd = dup(content_fd);
	if (fd == -1) {
		free(mm_boundary);
		return true;
	}

	FILE *stream = fdopen(fd, "r");
	if (!stream) {
		close(fd);
		lseek(content_fd, offset, SEEK_SET);
		free(mm_boundary);
		return true;
	}

	if (skip_prologue(stream, mm_boundary)) {
		fclose(stream);
		lseek(content_fd, offset, SEEK_SET);
		free(mm_boundary);
		return true;
	}

	char *dir = signat_conf_dir();
	if (!dir) {
		fclose(stream);
		lseek(content_fd, offset, SEEK_SET);
		free(mm_boundary);
		return true;
	}

	bool error = smime_check(sender, stream, micalg, mm_boundary, dir);
	free(dir);
	free(mm_boundary);

	if (fclose(stream))
		error = true;
	if (lseek(content_fd, offset, SEEK_SET) == -1)
		error = true;

	return error;
}

static bool skip_prologue(FILE *stream, const char *mm_boundary) {
	int state = 0;
	int i = 0;
	int c = getc(stream);
	while (c != EOF) {
		switch (state) {
		case 0:
			if (c == '\n')
				state++;
			break;
		case 1:
			if (c == mm_boundary[i]) {
				if (!mm_boundary[++i])
					state++;
			}
			else {
				state = 0;
				i = 0;
				continue;
			}
			break;
		case 2:
			if (c == '\r') {
				state++;
				break;
			}
		case 3:
			return c != '\n';
		}
		c = getc(stream);
	}
	return true;
}
