// Whitelist check

#define _GNU_SOURCE

#include "checks.h"

#include <stdbool.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <exim4-signat/common.h>

static int skip_line(int c, FILE *stream);

static bool find_address(const char *address, const char *dir,
	const char *file) {
	const char *at = strchr(address, '@');
	if (!at++)
		return false;

	char *path;
	if (asprintf(&path, "%s/%s", dir, file) == -1)
		return false;
	FILE *stream = fopen(path, "r");
	free(path);
	if (!stream)
		return false;

	bool match = false;
	int c = getc(stream);
	while (c == '\n')
		c = getc(stream);
	for (; c != EOF; c = skip_line(c, stream)) {
		if (c == '#')
			continue;
		const char *ac;
		if (c != '@') {
			ac = address;
			while (*ac != '@' && *ac == c) {
				ac++;
				c = getc(stream);
			}
			if (*ac != c)
				continue;
		}
		ac = at;
		c = getc(stream);
		while (*ac && c != EOF && to_c_lower(*ac) == to_c_lower(c)) {
			ac++;
			c = getc(stream);
		}
		if (!*ac && (c == '\n' || c == EOF)) {
			match = true;
			break;
		}
	}

	if (fclose(stream))
		return false;
	return match;
}

bool signat_whitelist_check(const char *recipient, const char *sender) {
	char *dir = signat_conf_dir();
	if (!dir)
		return true;

	bool error = !find_address(recipient, dir, "whitelist")
		&& !find_address(sender, dir, "whitelist-from");
	free(dir);
	return error;
}

static int skip_line(int c, FILE *stream) {
	while (c != '\n' && c != EOF)
		c = getc(stream);
	while (c == '\n')
		c = getc(stream);
	return c;
}
